﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.UI;

namespace SchedulerCustomAppointmentDB
{
    public partial class Form1 : Form
    {
        private TimeSpan startNotification = new TimeSpan(0, 15, 0); 
        
        public Form1()
        {
            InitializeComponent();

            this.schedulerBindingDataSource1.EventProvider.AppointmentFactory = new CustomAppointmentFactory();
            this.radScheduler1.AppointmentFactory = new CustomAppointmentFactory();

            AppointmentMappingInfo appointmentMappingInfo = new AppointmentMappingInfo();
            appointmentMappingInfo.BackgroundId = "BackgroundID";
            appointmentMappingInfo.Description = "Description";
            appointmentMappingInfo.End = "End";
            appointmentMappingInfo.Location = "Location";
            appointmentMappingInfo.MasterEventId = "ParentID";
            appointmentMappingInfo.RecurrenceRule = "RecurrenceRule";
            appointmentMappingInfo.ResourceId = "ResourceID";
            appointmentMappingInfo.Resources = "Appointments_AppointmentsResources";
            appointmentMappingInfo.Start = "Start";
            appointmentMappingInfo.StatusId = "StatusID";
            appointmentMappingInfo.Summary = "Summary";
            schedulerBindingDataSource1.EventProvider.Mapping = appointmentMappingInfo;
            
            (this.schedulerBindingDataSource1.EventProvider.Mapping as AppointmentMappingInfo).Mappings.Add(
                new SchedulerMapping("Email", "Email"));
            this.schedulerBindingDataSource1.EventProvider.Mapping.FindBySchedulerProperty("RecurrenceRule").ConvertToScheduler = new ConvertCallback(ConvertRecurrenceToScheduler);
            this.schedulerBindingDataSource1.EventProvider.Mapping.FindBySchedulerProperty("RecurrenceRule").ConvertToDataSource = new ConvertCallback(ConvertRecurrenceToDataSource);
   
            ResourceMappingInfo resourceMappingInfo = new ResourceMappingInfo();
            resourceMappingInfo.Id = "ID";
            resourceMappingInfo.Name = "ResourceName";
            this.schedulerBindingDataSource1.ResourceProvider.Mapping = resourceMappingInfo;

            this.resourcesTableAdapter.Fill(this.dataFlowSchedDataSet.Resources);
            this.appointmentsResourcesTableAdapter.Fill(this.dataFlowSchedDataSet.AppointmentsResources);
            this.appointmentsTableAdapter.Fill(this.dataFlowSchedDataSet.Appointments);

            schedulerBindingDataSource1.ResourceProvider.DataSource = dataFlowSchedDataSet.Resources;
            schedulerBindingDataSource1.EventProvider.DataSource = dataFlowSchedDataSet.Appointments;

            radScheduler1.DataSource = schedulerBindingDataSource1; 
            
            this.radSchedulerReminder1.StartReminder();
            this.radSchedulerReminder1.ItemOpened += radSchedulerReminder1_ItemOpened;
        }

        private void radSchedulerReminder1_ItemOpened(object sender, RadOpenItemArgs e)
        {
            CustomAppointmentEditForm editAppointmentDialog = new CustomAppointmentEditForm((IEvent)e.RemindObject, this.radScheduler1);
            ThemeResolutionService.ApplyThemeToControlTree(editAppointmentDialog, this.radScheduler1.ThemeName);
            editAppointmentDialog.ShowDialog();
        }

        public object ConvertRecurrenceToScheduler(object item)
        {
            string value = Convert.ToString(item);
            
            if ((string.IsNullOrEmpty(value)))
            {
                return null;
            }

            if (value.Contains("RRULE:"))
            {
                value = value.Substring(value.IndexOf("RRULE:") + "RRULE:".Length);

                if ((value.Contains("BYDAY") & value.Contains("FREQ=DAILY")))
                {
                    value = value.Replace("FREQ=DAILY", "FREQ=WEEKLY");
                }

                value = value.TrimEnd('\n', '\r', ';');
            }

            RecurrenceRule rule = default(RecurrenceRule);

            Telerik.WinControls.UI.Scheduler.ICalendar.CalHelper.TryParseRecurrenceRule(value, out rule);

            return rule;
        }

        public object ConvertRecurrenceToDataSource(object item)
        {
            //this is just a sample string conversion
            RecurrenceRule rule = default(RecurrenceRule);

            foreach (IEvent ev in this.radScheduler1.Appointments)
            {
                if (ev.RecurrenceRule != null && ev.RecurrenceRule.Equals(item))
                {
                    rule = (RecurrenceRule)item;

                    if ((rule != null))
                    {
                        string res = rule.ToString();

                        if ((!res.Contains("INTERVAL")))
                        {
                            res += ";INTERVAL=1";
                        }

                        res += ";" + Environment.NewLine;
                        res = "DTSTART:" + ev.Start.ToString("yyyyMMdd\\THHmmss\\Z") +
                              Environment.NewLine + "DTEND:" + ev.End.ToString("yyyyMMdd\\THHmmss\\Z") +
                              Environment.NewLine + "RRULE:" + res;

                        return res;
                    }
                }
            }

            return DBNull.Value;
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            // TODO: This line of code loads data into the 'dataFlowSchedDataSet.Resources' table. You can move, or remove it, as needed.
            this.resourcesTableAdapter.Fill(this.dataFlowSchedDataSet.Resources);
            // TODO: This line of code loads data into the 'dataFlowSchedDataSet.AppointmentsResources' table. You can move, or remove it, as needed.
            this.appointmentsResourcesTableAdapter.Fill(this.dataFlowSchedDataSet.AppointmentsResources);
            // TODO: This line of code loads data into the 'dataFlowSchedDataSet.Appointments' table. You can move, or remove it, as needed.
            this.appointmentsTableAdapter.Fill(this.dataFlowSchedDataSet.Appointments);
        }

        private void radButton1_Click(object sender, EventArgs e)
        {
            SaveScheduler();
        }
  
        public void SaveScheduler()
        {
            appointmentsResourcesTableAdapter.Adapter.AcceptChangesDuringUpdate = false;
            DataFlowSchedDataSet.AppointmentsResourcesDataTable deletedChildRecords =
                this.dataFlowSchedDataSet.AppointmentsResources.GetChanges(DataRowState.Deleted)
                as DataFlowSchedDataSet.AppointmentsResourcesDataTable;
            DataFlowSchedDataSet.AppointmentsResourcesDataTable newChildRecords =
                this.dataFlowSchedDataSet.AppointmentsResources.GetChanges(DataRowState.Added)
                as DataFlowSchedDataSet.AppointmentsResourcesDataTable;
            DataFlowSchedDataSet.AppointmentsResourcesDataTable modifiedChildRecords =
                this.dataFlowSchedDataSet.AppointmentsResources.GetChanges(DataRowState.Modified)
                as DataFlowSchedDataSet.AppointmentsResourcesDataTable;
            try
            {
                if (deletedChildRecords != null)
                {
                    appointmentsResourcesTableAdapter.Update(deletedChildRecords);
                }
                appointmentsTableAdapter.Update(this.dataFlowSchedDataSet.Appointments);
                if (newChildRecords != null)
                {
                    appointmentsResourcesTableAdapter.Update(newChildRecords);
                }
                if (modifiedChildRecords != null)
                {
                    appointmentsResourcesTableAdapter.Update(modifiedChildRecords);
                }
                this.dataFlowSchedDataSet.AcceptChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("An error occurred during the update process:\n{0}", ex.Message));
            }
            finally
            {
                if (deletedChildRecords != null)
                {
                    deletedChildRecords.Dispose();
                }
                if (newChildRecords != null)
                {
                    newChildRecords.Dispose();
                }
                if (modifiedChildRecords != null)
                {
                    modifiedChildRecords.Dispose();
                }
            }
        }

        private void radScheduler1_AppointmentEditDialogShowing(object sender, AppointmentEditDialogShowingEventArgs e)
        {
            e.AppointmentEditDialog = new CustomAppointmentEditForm(this);
        }
    }

    class AppointmentWithEmail : Appointment
    {
        private string email = string.Empty;

        public string Email
        {
            get
            {
                return this.email;
            }
            set
            {
                if (this.email != value)
                {
                    this.email = value.Trim();
                    this.OnPropertyChanged("Email");
                }
            }
        }

        protected override Event CreateOccurrenceInstance()
        {
            AppointmentWithEmail occurrence = new AppointmentWithEmail();
            occurrence.email = this.email;
            return occurrence;
        }
    }

    class CustomAppointmentFactory : IAppointmentFactory
    {
        public IEvent CreateNewAppointment()
        {
            return new AppointmentWithEmail();
        }
    }
}